#!/usr/bin/env python

from collections import namedtuple
from configparser import SafeConfigParser
from io import StringIO
from subprocess import Popen, PIPE
import os
import re

from bottle import route, run, view


__version__ = '0.1'
__author__ = 'Josh VanderLinden <arch@cloudlery.com>'

CFG_RE = re.compile(r'^(?P<var>\w+) +(?:"(?P<sval>.*?)"|(?P<nval>\d+))$', re.M)
GRAPH_MAP = {
    'summary': 'vs',
    'hourly': 'h',
    'daily': 'd',
    'monthly': 'm',
    'top': 't',
}


def load_ui_config():
    """Load the UI configuration into a usable object"""

    cfg = SafeConfigParser({
        'interfaces': '',
        'graphs': 'summary,hourly,daily,monthly,top',
        'title': 'Traffic Summary',
        'show_license': 'true',
        'bind': '0.0.0.0',
        'port': '7000',
    })
    cfg.read(['/etc/conf.d/vnstatui.conf'])
    cfg.getlist = lambda s, o: cfg.get(s, o).split(',')
    if not cfg.has_section('vnstatui'):
        cfg.add_section('vnstatui')

    return cfg


def load_vnstat_config(config='/etc/vnstat.conf'):
    """Load vnstat configuration into a usable object"""

    settings = {}
    with open(config) as fh:
        contents = fh.read()
        matches = CFG_RE.findall(contents)

        for key, sval, nval in matches:
            if nval:
                settings[key] = int(nval)
            else:
                settings[key] = sval

    return namedtuple('VNStatConfig', settings.keys())(**settings)


@route('/')
@view('index')
def index():
    """Generate the page to display the graphs"""

    cfg = load_ui_config()
    vnstat_config = load_vnstat_config()
    known_interfaces = [f for f in os.listdir(vnstat_config.DatabaseDir)
                        if not f.startswith('.')]

    graphs = cfg.getlist('vnstatui', 'graphs')
    interfaces = list(filter(lambda v: v in known_interfaces,
                             cfg.getlist('vnstatui', 'interfaces')))
    if not interfaces:
        interfaces = known_interfaces

    return {
        'interfaces': interfaces,
        'graphs': graphs,
        'title': cfg.get('vnstatui', 'title'),
        'show_license': cfg.getboolean('vnstatui', 'show_license'),
        'version': __version__,
    }


@route('/graph/:interface/:graph')
def make_graph(interface, graph):
    """Generate a graph for the specified interface"""

    flag = GRAPH_MAP[graph]
    cmd = ['/usr/bin/vnstati',
           '-%s' % flag,
           '-i', interface,
           '-o', '-']
    pipe = Popen(cmd, stdout=PIPE)
    (img, _) = pipe.communicate()
    return img


@route('/license/')
def license():
    """Return the license for this project"""

    from docutils.core import publish_file

    # look in the same directory first
    app_dir = os.path.abspath(os.path.dirname(__file__))
    license_path = os.path.join(app_dir, 'LICENSE')
    if not os.path.exists(license_path):
        # fall back to the standard license location
        license_path = '/usr/share/licenses/vnstatui/LICENSE'

    lic = ''
    with open(license_path) as fh:
        lic = publish_file(fh, writer_name='html', destination=StringIO())

    return lic


def serve(bind=None, port=None):
    """Run the server and wait for requests"""

    cfg = load_ui_config()

    if bind is None:
        bind = cfg.get('vnstatui', 'bind')

    if port is None:
        port = cfg.getint('vnstatui', 'port')

    run(host=bind, port=port)


if __name__ == '__main__':
    serve()
